@@ -192,16 +192,19 @@ AnthyInstance::process_key_event (const KeyEvent& key)
 {
     SCIM_DEBUG_IMENGINE(2) << "process_key_event.\n";
 
-    if (process_key_event_input (key))
-        return true;
+    // FIXME!
+    // for NICOLA thumb shift key
+    if (m_preedit.get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA &&
+        is_nicola_thumb_shift_key (key))
+    {
+        if (process_key_event_input (key))
+            return true;
+    }
 
     // lookup user defined key bindings
     if (process_key_event_lookup_keybind (key))
         return true;
 
-    if (m_preedit.get_input_mode () == SCIM_ANTHY_MODE_LATIN)
-        return false;
-
     // process hard coded keys
     if (is_selecting_candidates () && m_lookup_table_visible)
         if (process_key_event_with_candidate (key))
@@ -213,6 +216,18 @@ AnthyInstance::process_key_event (const KeyEvent& key)
         if (process_key_event_without_preedit(key))
             return true;
 
+    // for Multi/Dead key
+    if (m_preedit.get_input_mode () == SCIM_ANTHY_MODE_LATIN)
+        return false;
+
+    // for input key event
+    if (m_preedit.get_typing_method () != SCIM_ANTHY_TYPING_METHOD_NICOLA ||
+        !is_nicola_thumb_shift_key (key))
+    {
+        if (process_key_event_input (key))
+            return true;
+    }
+
     if (m_preedit.is_preediting ())
         return true;
     else
@@ -1683,6 +1698,18 @@ AnthyInstance::reload_config (const ConfigPointer &config)
             m_preedit.set_typing_method
                 (SCIM_ANTHY_TYPING_METHOD_ROMAJI,
                  m_factory->m_custom_romaji_table);
+    } else {
+        if (m_preedit.get_typing_method () == SCIM_ANTHY_TYPING_METHOD_NICOLA)
+            m_preedit.set_typing_method
+                (SCIM_ANTHY_TYPING_METHOD_NICOLA, NULL);
+        else if (m_preedit.get_typing_method () == SCIM_ANTHY_TYPING_METHOD_KANA)
+            m_preedit.set_typing_method
+                (SCIM_ANTHY_TYPING_METHOD_KANA,
+                 m_factory->m_custom_kana_table);
+        else
+            m_preedit.set_typing_method
+                (SCIM_ANTHY_TYPING_METHOD_ROMAJI,
+                 m_factory->m_custom_romaji_table);
     }
 
     // set conversion mode
