@@ -25,28 +25,31 @@ using namespace scim_anthy;
 static Key2KanaTable romaji_table (
     utf8_mbstowcs ("DefaultRomajiTable"),
     scim_anthy_romaji_typing_rule);
-static Key2KanaTable romaji_consonant_table (
-    utf8_mbstowcs ("DefaultRomajiConsonantTable"),
-    scim_anthy_romaji_consonant_rule);
+static Key2KanaTable romaji_double_consonant_table (
+    utf8_mbstowcs ("DefaultRomajiDoubleConsonantTable"),
+    scim_anthy_romaji_double_consonant_rule);
 static Key2KanaTable kana_table (
     utf8_mbstowcs ("DefaultKanaTable"),
     scim_anthy_kana_typing_rule);
+static Key2KanaTable kana_voiced_consonant_table (
+    utf8_mbstowcs ("DefaultKanaVoicedConsonantTable"),
+    scim_anthy_kana_voiced_consonant_rule);
 
 // symbols
 static Key2KanaTable half_symbol_table (
     utf8_mbstowcs ("DefaultRomajiHalfSymbolTable"),
-    scim_anthy_romaji_half_symbol_rule);
+    scim_anthy_half_symbol_rule);
 static Key2KanaTable wide_symbol_table (
     utf8_mbstowcs ("DefaultRomajiWideSymbolTable"),
-    scim_anthy_romaji_wide_symbol_rule);
+    scim_anthy_wide_symbol_rule);
 
 // numbers
 static Key2KanaTable half_number_table (
     utf8_mbstowcs ("DefaultRomajiHalfNumberTable"),
-    scim_anthy_romaji_half_number_rule);
+    scim_anthy_half_number_rule);
 static Key2KanaTable wide_number_table (
     utf8_mbstowcs ("DefaultRomajiWideNumberTable"),
-    scim_anthy_romaji_wide_number_rule);
+    scim_anthy_wide_number_rule);
 
 // period
 static Key2KanaTable romaji_ja_period_table (
@@ -131,11 +134,9 @@ Key2KanaTable::Key2KanaTable (WideString name, ConvRule *table)
     : m_name (name)
 {
     for (unsigned int i = 0; table[i].string; i++) {
-        m_rules.push_back (
-            Key2KanaRule (
-                table[i].string ? table[i].string : "",
-                table[i].result ? table[i].result : "",
-                table[i].cont   ? table[i].cont   : ""));
+        append_rule (table[i].string ? table[i].string : "",
+                     table[i].result ? table[i].result : "",
+                     table[i].cont   ? table[i].cont   : "");
     }
 }
 
@@ -143,14 +144,32 @@ Key2KanaTable::~Key2KanaTable ()
 {
 }
 
+void
+Key2KanaTable::append_rule (String sequence,
+                            String result,
+                            String cont)
+{
+    // FIXME! check duplicates
+    m_rules.push_back (Key2KanaRule (sequence, result, cont));
+}
+
+void
+Key2KanaTable::clear (void)
+{
+    m_rules.clear ();
+
+}
 
 Key2KanaTableSet::Key2KanaTableSet ()
-    : m_name            (utf8_mbstowcs ("")),
-      m_typing_method   (SCIM_ANTHY_TYPING_METHOD_ROMAJI),
-      m_period_style    (SCIM_ANTHY_PERIOD_JAPANESE),
-      m_comma_style     (SCIM_ANTHY_COMMA_JAPANESE),
-      m_use_half_symbol (false),
-      m_use_half_number (false)
+    : m_name                   (utf8_mbstowcs ("")),
+      m_fundamental_table      (NULL),
+      m_voiced_consonant_table (Key2KanaTable (utf8_mbstowcs ("voiced consonant table"))),
+      m_additional_table       (NULL),
+      m_typing_method          (SCIM_ANTHY_TYPING_METHOD_ROMAJI),
+      m_period_style           (SCIM_ANTHY_PERIOD_JAPANESE),
+      m_comma_style            (SCIM_ANTHY_COMMA_JAPANESE),
+      m_use_half_symbol        (false),
+      m_use_half_number        (false)
 {
     set_typing_method (m_typing_method);
 }
@@ -196,6 +215,54 @@ Key2KanaTableSet::set_comma_style (CommaStyle  style)
     reset_tables ();
 }
 
+static void
+create_voiced_consonant_table (Key2KanaTable &table, Key2KanaTable &fund_table)
+{
+    table.clear ();
+
+    const String sonant_mark      = String ("\xE3\x82\x9B");
+    const String half_sonant_mark = String ("\xE3\x82\x9C");
+    std::vector<String> sonant_mark_list;
+    std::vector<String> half_sonant_mark_list;
+
+    Key2KanaRules::iterator it;
+    Key2KanaRules &rules = fund_table.get_table ();
+    for (it = rules.begin (); it != rules.end (); it++) {
+        String result = it->get_result ();
+        if (result == sonant_mark)
+            sonant_mark_list.push_back (it->get_sequence ());
+        else if (result == half_sonant_mark)
+            half_sonant_mark_list.push_back (it->get_sequence ());
+    }
+
+    VoicedConsonantRule *templ = scim_anthy_voiced_consonant_table;
+
+    for (unsigned int i = 0; templ[i].string; i++) {
+        if (templ[i].voiced && *templ[i].voiced) {
+            std::vector<String>::iterator it;
+            for (it = sonant_mark_list.begin ();
+                 it != sonant_mark_list.end ();
+                 it++)
+            {
+                table.append_rule (String (templ[i].string) + *it,
+                                   String (templ[i].voiced),
+                                   String ());
+            }
+        }
+        if (templ[i].half_voiced && *templ[i].half_voiced) {
+            std::vector<String>::iterator it;
+            for (it = half_sonant_mark_list.begin ();
+                 it != half_sonant_mark_list.end ();
+                 it++)
+            {
+                table.append_rule (String (templ[i].string) + *it,
+                                   String (templ[i].half_voiced),
+                                   String ());
+            }
+        }
+    }
+}
+
 void
 Key2KanaTableSet::reset_tables (void)
 {
@@ -204,19 +271,17 @@ Key2KanaTableSet::reset_tables (void)
     bool is_romaji = m_typing_method == SCIM_ANTHY_TYPING_METHOD_ROMAJI;
     bool is_kana   = m_typing_method == SCIM_ANTHY_TYPING_METHOD_KANA;
 
-    if (is_romaji) {
-        // symbols table
-        if (m_use_half_symbol)
-            m_all_tables.push_back (&half_symbol_table);
-        else
-            m_all_tables.push_back (&wide_symbol_table);
+    // symbols table
+    if (m_use_half_symbol)
+        m_all_tables.push_back (&half_symbol_table);
+    else
+        m_all_tables.push_back (&wide_symbol_table);
 
-        // numbers table
-        if (m_use_half_number)
-            m_all_tables.push_back (&half_number_table);
-        else
-            m_all_tables.push_back (&wide_number_table);
-    }
+    // numbers table
+    if (m_use_half_number)
+        m_all_tables.push_back (&half_number_table);
+    else
+        m_all_tables.push_back (&wide_number_table);
 
     if (is_romaji || is_kana)
     {
@@ -272,9 +337,12 @@ Key2KanaTableSet::reset_tables (void)
 
     if (!m_fundamental_table) {
         if (is_romaji) {
-            m_all_tables.push_back (&romaji_consonant_table);
+            m_all_tables.push_back (&romaji_double_consonant_table);
             m_all_tables.push_back (&romaji_table);
         } else if (is_kana) {
+            create_voiced_consonant_table (m_voiced_consonant_table,
+                                           kana_table);
+            m_all_tables.push_back (&m_voiced_consonant_table);
             m_all_tables.push_back (&kana_table);
         }
     } else {
@@ -284,9 +352,12 @@ Key2KanaTableSet::reset_tables (void)
             m_all_tables.push_back (&(*it));
 #else
         if (is_romaji) {
-            m_all_tables.push_back (&romaji_consonant_table);
+            m_all_tables.push_back (&romaji_double_consonant_table);
             m_all_tables.push_back (m_fundamental_table);
         } else if (is_kana) {
+            create_voiced_consonant_table (m_voiced_consonant_table,
+                                           *m_fundamental_table);
+            m_all_tables.push_back (&m_voiced_consonant_table);
             m_all_tables.push_back (m_fundamental_table);
         }
 #endif
