@@ -21,6 +21,8 @@
   #include <config.h>
 #endif
 
+#include "scim_anthy_factory.h"
+#include "scim_anthy_imengine.h"
 #include "scim_anthy_preedit.h"
 #include "scim_anthy_utils.h"
 
@@ -31,10 +33,11 @@ static ConvRule *get_period_rule              (TypingMethod method,
 static ConvRule *get_comma_rule               (TypingMethod method,
                                                CommaStyle   period);
 
-Preedit::Preedit (Key2KanaTableSet & tables)
-    : m_key2kana_tables    (tables),
-      m_reading            (tables),
-      m_conversion         (m_reading),
+Preedit::Preedit (AnthyInstance &anthy, Key2KanaTableSet & tables)
+    : m_anthy              (anthy),
+      m_key2kana_tables    (tables),
+      m_reading            (anthy, tables),
+      m_conversion         (m_anthy, m_reading),
       m_input_mode         (SCIM_ANTHY_MODE_HIRAGANA),
       m_behavior_on_period (SCIM_ANTHY_NONE_ON_PERIOD)
 {
@@ -68,18 +71,18 @@ Preedit::get_string (void)
         WideString widestr;
         switch (m_input_mode) {
         case SCIM_ANTHY_MODE_KATAKANA:
-            convert_to_katakana (widestr, m_reading.get ());
+            util_convert_to_katakana (widestr, m_reading.get ());
             return widestr;
 
         case SCIM_ANTHY_MODE_HALF_KATAKANA:
-            convert_to_katakana (widestr, m_reading.get (), true);
+            util_convert_to_katakana (widestr, m_reading.get (), true);
             return widestr;
 
         case SCIM_ANTHY_MODE_LATIN:
             return utf8_mbstowcs (m_reading.get_raw ());
 
         case SCIM_ANTHY_MODE_WIDE_LATIN:
-            convert_to_wide (widestr, m_reading.get_raw ());
+            util_convert_to_wide (widestr, m_reading.get_raw ());
             return widestr;
 
         case SCIM_ANTHY_MODE_HIRAGANA:
@@ -95,9 +98,32 @@ AttributeList
 Preedit::get_attribute_list (void)
 {
     if (is_converting ())
+    {
         return m_conversion.get_attribute_list ();
-    else
-        return AttributeList ();
+    } else {
+        AttributeList attrs;
+        util_create_attributes (attrs, 0, get_length (),
+                                m_anthy.get_factory()->m_preedit_style,
+                                m_anthy.get_factory()->m_preedit_fg_color,
+                                m_anthy.get_factory()->m_preedit_bg_color);
+#if 0
+        attrs.push_back (
+            Attribute (0, get_length (),
+                       SCIM_ATTR_FOREGROUND, 
+                       m_anthy.get_factory()->m_preedit_fg_color));
+        attrs.push_back (
+            Attribute (0, get_length (),
+                       SCIM_ATTR_BACKGROUND,
+                       m_anthy.get_factory()->m_preedit_bg_color));
+#endif
+        return attrs;
+    }
+}
+
+Reading &
+Preedit::get_reading (void)
+{
+    return m_reading;
 }
 
 bool
@@ -417,7 +443,6 @@ Preedit::is_comma_or_period (const String & str)
 }
 
 
-
 /*
  * utilities
  */
