@@ -27,6 +27,16 @@ namespace scim_anthy {
 
 BoolConfigData config_bool_common [] =
 {
+    {
+        SCIM_ANTHY_CONFIG_SHOW_CANDIDATES_LABEL,
+        SCIM_ANTHY_CONFIG_SHOW_CANDIDATES_LABEL_DEFAULT,
+        SCIM_ANTHY_CONFIG_SHOW_CANDIDATES_LABEL_DEFAULT,
+        N_("Show \"_Candidates\" label"),
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
     {
         SCIM_ANTHY_CONFIG_CLOSE_CAND_WIN_ON_SELECT,
         SCIM_ANTHY_CONFIG_CLOSE_CAND_WIN_ON_SELECT_DEFAULT,
@@ -168,6 +178,58 @@ BoolConfigData config_bool_common [] =
     },
 };
 
+IntConfigData config_int_common [] =
+{
+    {
+        SCIM_ANTHY_CONFIG_CAND_WIN_PAGE_SIZE,
+        SCIM_ANTHY_CONFIG_CAND_WIN_PAGE_SIZE_DEFAULT,
+        SCIM_ANTHY_CONFIG_CAND_WIN_PAGE_SIZE_DEFAULT,
+        1, 100, 1,
+        N_("Number of candidates to show in a _page:"),
+        NULL,
+        NULL,
+        N_("Specify the number of candidates to show in a page of the candidates window."),
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_N_TRIGGERS_TO_SHOW_CAND_WIN,
+        SCIM_ANTHY_CONFIG_N_TRIGGERS_TO_SHOW_CAND_WIN_DEFAULT,
+        SCIM_ANTHY_CONFIG_N_TRIGGERS_TO_SHOW_CAND_WIN_DEFAULT,
+        0, 100, 1,
+        N_("Number of _triggers until show:"),
+        N_("[times]"),
+        NULL,
+        N_("The number of times to press a conversion key until show the candidates window."
+           "Specify 0 to never show it."),
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_NICOLA_TIME,
+        SCIM_ANTHY_CONFIG_NICOLA_TIME_DEFAULT,
+        SCIM_ANTHY_CONFIG_NICOLA_TIME_DEFAULT,
+        0, 10000, 1,
+        N_(""),
+        N_("[msec]"),
+        NULL,
+        N_(""),
+        NULL,
+        false,
+    },
+    {
+        NULL,
+        0,
+        0,
+        0, 0, 0,
+        NULL,
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
+};
+
 StringConfigData config_string_common [] =
 {
     {
@@ -240,6 +302,26 @@ StringConfigData config_string_common [] =
         NULL,
         false,
     },
+    {
+        SCIM_ANTHY_CONFIG_LEFT_THUMB_SHIFT_KEY,
+        SCIM_ANTHY_CONFIG_LEFT_THUMB_SHIFT_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_LEFT_THUMB_SHIFT_KEY_DEFAULT,
+        N_("_Left thumb shift keys:"),
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_RIGHT_THUMB_SHIFT_KEY,
+        SCIM_ANTHY_CONFIG_RIGHT_THUMB_SHIFT_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_RIGHT_THUMB_SHIFT_KEY_DEFAULT,
+        N_("_Right thumb shift keys:"),
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
     {
         SCIM_ANTHY_CONFIG_DICT_ADMIN_COMMAND,
         SCIM_ANTHY_CONFIG_DICT_ADMIN_COMMAND_DEFAULT,
@@ -260,6 +342,36 @@ StringConfigData config_string_common [] =
         NULL,
         false,
     },
+    {
+        SCIM_ANTHY_CONFIG_PREEDIT_STYLE,
+        SCIM_ANTHY_CONFIG_PREEDIT_STYLE_DEFAULT,
+        SCIM_ANTHY_CONFIG_PREEDIT_STYLE_DEFAULT,
+        N_("_Preedit string:"),
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_CONVERSION_STYLE,
+        SCIM_ANTHY_CONFIG_CONVERSION_STYLE_DEFAULT,
+        SCIM_ANTHY_CONFIG_CONVERSION_STYLE_DEFAULT,
+        N_("_Conversion string:"),
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_STYLE,
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_STYLE_DEFAULT,
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_STYLE_DEFAULT,
+        N_("_Selected segment:"),
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
     {
         NULL,
         "",
@@ -364,6 +476,17 @@ StringConfigData config_keyboards_edit [] =
         NULL,
         false,
     },
+    {
+        SCIM_ANTHY_CONFIG_DO_NOTHING_KEY,
+        SCIM_ANTHY_CONFIG_DO_NOTHING_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_DO_NOTHING_KEY_DEFAULT,
+        N_("Do nothing"),
+        N_("Select do nothing keys"),
+        N_("The key events to eat and do nothing anymore. "
+           "For example, it can be used to disable space key completely."),
+        NULL,
+        false,
+    },
     {
         NULL,
         "",
@@ -941,4 +1064,59 @@ StringConfigData config_keyboards_reverse_learning [] =
     },
 };
 
+ColorConfigData config_color_common [] =
+{
+    {
+        SCIM_ANTHY_CONFIG_PREEDIT_FG_COLOR,
+        SCIM_ANTHY_CONFIG_PREEDIT_FG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_PREEDIT_FG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_PREEDIT_BG_COLOR,
+        SCIM_ANTHY_CONFIG_PREEDIT_BG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_PREEDIT_BG_COLOR_DEFAULT,
+        N_("Color:"),
+        N_("The color of preediting text"),
+        N_("The color of preediting text"),
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_CONVERSION_FG_COLOR,
+        SCIM_ANTHY_CONFIG_CONVERSION_FG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_CONVERSION_FG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_CONVERSION_BG_COLOR,
+        SCIM_ANTHY_CONFIG_CONVERSION_BG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_CONVERSION_BG_COLOR_DEFAULT,
+        N_("Color:"),
+        N_("The color of conversion text"),
+        N_("The color of conversion text"),
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_FG_COLOR,
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_FG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_FG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_BG_COLOR,
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_BG_COLOR_DEFAULT,
+        SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_BG_COLOR_DEFAULT,
+        N_("Color:"),
+        N_("The color of selected segment text"),
+        N_("The color of selected segment text in the conversion text"),
+        NULL,
+        false,
+    },
+    {
+        NULL,
+        "",
+        "",
+        NULL,
+        "",
+        "",
+        NULL,
+        NULL,
+        NULL,
+        NULL,
+        false,
+    },
+};
 }
