@@ -39,8 +39,11 @@
 #include "scim_anthy_style_file.h"
 #include "scim_anthy_prefs.h"
 #include "scim_anthy_default_tables.h"
+#include "scim_anthy_setup.h"
 #include "scim_anthy_setup_romaji.h"
+#include "scim_anthy_setup_kana.h"
 #include "scim_anthy_utils.h"
+#include "scim_color_button.h"
 
 using namespace scim;
 using namespace scim_anthy;
@@ -172,7 +175,8 @@ static GtkWidget   * __widget_key_list_view         = NULL;
 static GtkWidget   * __widget_choose_keys_button    = NULL;
 static GtkTooltips * __widget_tooltips              = NULL;
 
-static String __config_key_theme    = SCIM_ANTHY_CONFIG_KEY_THEME_DEFAULT;
+static String __config_key_theme      = SCIM_ANTHY_CONFIG_KEY_THEME_DEFAULT;
+static String __config_key_theme_file = SCIM_ANTHY_CONFIG_KEY_THEME_FILE_DEFAULT;
 
 static struct KeyboardConfigPage __key_conf_pages[] =
 {
@@ -202,8 +206,9 @@ static ComboConfigCandidate input_modes[] =
 
 static ComboConfigCandidate typing_methods[] =
 {
-    {N_("Romaji typing method"), "Roma"},
-    {N_("Kana typing method"),   "Kana"},
+    {N_("Romaji typing method"),      "Romaji"},
+    {N_("Kana typing method"),        "Kana"},
+    {N_("Thumb shift typing method"), "NICOLA"},
     {NULL, NULL},
 };
 
@@ -250,6 +255,19 @@ static ComboConfigCandidate behavior_on_period[] =
 };
 
 
+static ComboConfigCandidate preedit_style[] =
+{
+    {N_("No decoration"), "None"},
+    {N_("Underline"),     "Underline"},
+    {N_("Reverse"),       "Reverse"},
+    {N_("Highlight"),     "Highlight"},
+    {N_("String color"),  "FGColor"},
+    {N_("BG color"),      "BGColor"},
+    {N_("Both color"),    "Color"},
+    {NULL, NULL},
+};
+
+
 static void     setup_key_theme_menu              (GtkOptionMenu *omenu);
 static void     setup_widget_value                (void);
 
@@ -257,10 +275,18 @@ static void     on_default_editable_changed       (GtkEditable      *editable,
                                                    gpointer          user_data);
 static void     on_default_toggle_button_toggled  (GtkToggleButton  *togglebutton,
                                                    gpointer          user_data);
+static void     on_default_spin_button_changed    (GtkSpinButton    *spinbutton,
+                                                   gpointer          user_data);
+#if 0
 static void     on_default_key_selection_clicked  (GtkButton        *button,
                                                    gpointer          user_data);
+#endif
 static void     on_default_combo_changed          (GtkEditable      *editable,
                                                    gpointer          user_data);
+static void     on_default_option_menu_changed    (GtkOptionMenu    *omenu,
+                                                   gpointer          user_data);
+static void     on_preedit_style_menu_changed     (GtkOptionMenu    *omenu,
+                                                   gpointer          user_data);
 static void     on_key_filter_selection_clicked   (GtkButton        *button,
                                                    gpointer          user_data);
 static void     on_dict_menu_label_toggled        (GtkToggleButton  *togglebutton,
@@ -281,6 +307,8 @@ static void     on_choose_keys_button_clicked     (GtkWidget        *button,
                                                    gpointer          data);
 static void     on_dict_launch_button_clicked     (GtkButton        *button,
                                                    gpointer          user_data);
+static void     on_color_button_changed           (ScimColorButton  *button,
+                                                   gpointer          user_data);
 
 
 static BoolConfigData *
@@ -298,7 +326,22 @@ find_bool_config_entry (const char *config_key)
     return NULL;
 }
 
-static StringConfigData *
+static IntConfigData *
+find_int_config_entry (const char *config_key)
+{
+    if (!config_key)
+        return NULL;
+
+    for (unsigned int i = 0; config_int_common[i].key; i++) {
+        IntConfigData *entry = &config_int_common[i];
+        if (entry->key && !strcmp (entry->key, config_key))
+            return entry;
+    }
+
+    return NULL;
+}
+
+StringConfigData *
 find_string_config_entry (const char *config_key)
 {
     if (!config_key)
@@ -327,16 +370,19 @@ find_key_config_entry (const char *config_key)
     return NULL;
 }
 
-static bool
-match_key_event (const KeyEventList &list, const KeyEvent &key)
+static ColorConfigData *
+find_color_config_entry (const char *config_key)
 {
-    KeyEventList::const_iterator kit;
+    if (!config_key)
+        return NULL;
 
-    for (kit = list.begin (); kit != list.end (); kit++) {
-        if (key.code == kit->code && key.mask == kit->mask)
-             return true;
+    for (unsigned int i = 0; config_color_common[i].fg_key; i++) {
+        ColorConfigData *entry = &config_color_common[i];
+        if (entry->fg_key && !strcmp (entry->fg_key, config_key))
+            return entry;
     }
-    return false;
+
+    return NULL;
 }
 
 GtkWidget *
@@ -362,15 +408,66 @@ create_check_button (const char *config_key)
     return GTK_WIDGET (entry->widget);
 }
 
-static void
-create_entry (StringConfigData *data, GtkTable *table, int i)
+void
+create_spin_button (const char *config_key, GtkTable *table, int idx)
+{
+    IntConfigData *entry = find_int_config_entry (config_key);
+    if (!entry)
+        return;
+
+    GtkWidget *label = gtk_label_new_with_mnemonic (_(entry->label));
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+    gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+    gtk_table_attach (GTK_TABLE (table), label, 0, 1, idx, idx + 1,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL),
+                      4, 4);
+    gtk_widget_show (GTK_WIDGET (label));
+
+    GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
+    gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (hbox),
+                      1, 2, idx, idx + 1,
+                      (GtkAttachOptions) GTK_FILL,
+                      (GtkAttachOptions) GTK_FILL,
+                      4, 4);
+    gtk_widget_show (hbox);
+
+    entry->widget = gtk_spin_button_new_with_range (entry->min, entry->max,
+                                                    entry->step);
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label),
+                                   GTK_WIDGET (entry->widget));
+    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (entry->widget),
+                        FALSE, FALSE, 0);
+    g_signal_connect (G_OBJECT (entry->widget), "value-changed",
+                      G_CALLBACK (on_default_spin_button_changed),
+                      entry);
+    gtk_widget_show (GTK_WIDGET (entry->widget));
+
+    if (entry->unit) {
+        label = gtk_label_new_with_mnemonic (_(entry->unit));
+        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+        gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+        gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (label),
+                            FALSE, FALSE, 0);
+        gtk_widget_show (GTK_WIDGET (label));
+    }
+
+    if (!__widget_tooltips)
+        __widget_tooltips = gtk_tooltips_new();
+    if (entry->tooltip)
+        gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (entry->widget),
+                              _(entry->tooltip), NULL);
+}
+
+GtkWidget *
+create_entry (StringConfigData *data, GtkTable *table, int idx)
 {
     GtkWidget *label = gtk_label_new (NULL);
     gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _(data->label));
     gtk_widget_show (label);
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-    gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i+1,
+    gtk_table_attach (GTK_TABLE (table), label, 0, 1, idx, idx + 1,
                       (GtkAttachOptions) (GTK_FILL),
                       (GtkAttachOptions) (GTK_FILL), 4, 4);
     (data)->widget = gtk_entry_new ();
@@ -381,7 +478,7 @@ create_entry (StringConfigData *data, GtkTable *table, int i)
                       data);
     gtk_widget_show (GTK_WIDGET (data->widget));
     gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (data->widget),
-                      1, 2, i, i+1,
+                      1, 2, idx, idx + 1,
                       (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                       (GtkAttachOptions) (GTK_FILL), 4, 4);
 
@@ -390,11 +487,13 @@ create_entry (StringConfigData *data, GtkTable *table, int i)
     if (data->tooltip)
         gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (data->widget),
                               _(data->tooltip), NULL);
+
+    return GTK_WIDGET (data->widget);
 }
 
 GtkWidget *
 create_combo (const char *config_key, gpointer candidates_p,
-              GtkWidget *table, gint idx)
+              GtkTable *table, gint idx)
 {
     StringConfigData *entry = find_string_config_entry (config_key);
     if (!entry)
@@ -439,6 +538,101 @@ create_combo (const char *config_key, gpointer candidates_p,
     return GTK_WIDGET (entry->widget);
 }
 
+GtkWidget *
+create_option_menu (const char *config_key, gpointer candidates_p,
+                    GtkTable *table, gint idx)
+{
+    StringConfigData *entry = find_string_config_entry (config_key);
+    ComboConfigCandidate *data = static_cast<ComboConfigCandidate*> (candidates_p);
+    if (!entry)
+        return NULL;
+
+    GtkWidget *label;
+    label = gtk_label_new_with_mnemonic (_(entry->label));
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+    gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+    gtk_table_attach (GTK_TABLE (table), label, 0, 1, idx, idx + 1,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL), 4, 4);
+    gtk_widget_show (label);
+
+    entry->widget = gtk_option_menu_new ();
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label),
+                                   GTK_WIDGET (entry->widget));
+    gtk_widget_show (GTK_WIDGET (entry->widget));
+    gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (entry->widget),
+                      1, 2, idx, idx + 1,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL), 4, 4);
+    g_object_set_data (G_OBJECT (entry->widget),
+                       DATA_POINTER_KEY,
+                       (gpointer) candidates_p);
+
+    GtkWidget *menu = gtk_menu_new ();
+    gtk_option_menu_set_menu (GTK_OPTION_MENU (entry->widget), menu);
+    gtk_widget_show (menu);
+
+    for (unsigned int i = 0; data[i].label; i++) {
+        GtkWidget *menuitem = gtk_menu_item_new_with_label (_(data[i].label));
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+        gtk_widget_show (menuitem);
+    }
+
+    gtk_option_menu_set_history (GTK_OPTION_MENU (entry->widget), 0);
+
+    g_signal_connect ((gpointer) GTK_OPTION_MENU (entry->widget), "changed",
+                      G_CALLBACK (on_default_option_menu_changed),
+                      entry);
+
+    if (!__widget_tooltips)
+        __widget_tooltips = gtk_tooltips_new();
+    if (entry->tooltip)
+        gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (entry->widget),
+                              _(entry->tooltip), NULL);
+
+    return GTK_WIDGET (entry->widget);
+}
+
+GtkWidget *
+create_color_button (const char *config_key)
+{
+    ColorConfigData *entry = find_color_config_entry (config_key);
+    if (!entry)
+        return NULL;
+
+    GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
+    gtk_widget_show (hbox);
+
+    GtkWidget *label = NULL;
+    if (entry->label) {
+        label = gtk_label_new_with_mnemonic (_(entry->label));
+        gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
+        gtk_widget_show (label);
+    }
+
+    entry->widget = scim_color_button_new ();
+    gtk_widget_set_size_request (GTK_WIDGET (entry->widget), 32, 24);
+    g_signal_connect (G_OBJECT (entry->widget), "color-changed",
+                      G_CALLBACK (on_color_button_changed),
+                      entry);
+    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (entry->widget),
+                        FALSE, FALSE, 2);
+    gtk_widget_show (GTK_WIDGET (entry->widget));
+
+    if (label)
+        gtk_label_set_mnemonic_widget (GTK_LABEL (label),
+                                       GTK_WIDGET (entry->widget));
+
+    if (!__widget_tooltips)
+        __widget_tooltips = gtk_tooltips_new();
+    if (entry->tooltip)
+        gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (entry->widget),
+                              _(entry->tooltip), NULL);
+
+    return hbox;
+}
+
 static void
 append_key_bindings (GtkTreeView *treeview, gint idx, const gchar *filter)
 {
@@ -459,7 +653,7 @@ append_key_bindings (GtkTreeView *treeview, gint idx, const gchar *filter)
             KeyEventList::const_iterator kit;
             bool found = true;
             for (kit = keys1.begin (); kit != keys1.end (); kit++) {
-                if (!match_key_event (keys2, *kit)) {
+                if (!util_match_key_event (keys2, *kit)) {
                     found = false;
                     break;
                 }
@@ -543,37 +737,37 @@ create_common_page (void)
     /* input mode */
     widget = create_combo (SCIM_ANTHY_CONFIG_INPUT_MODE,
                            (gpointer) &input_modes,
-                           table, 0);
+                           GTK_TABLE (table), 0);
 
     /* typing method */
     widget = create_combo (SCIM_ANTHY_CONFIG_TYPING_METHOD,
                            (gpointer) &typing_methods,
-                           table, 1);
+                           GTK_TABLE (table), 1);
 
     /* conversion mode */
     widget = create_combo (SCIM_ANTHY_CONFIG_CONVERSION_MODE,
                            (gpointer) &conversion_modes,
-                           table, 2);
+                           GTK_TABLE (table), 2);
 
     /* period style */
     widget = create_combo (SCIM_ANTHY_CONFIG_PERIOD_STYLE,
                            (gpointer) &period_styles,
-                           table, 3);
+                           GTK_TABLE (table), 3);
 
     /* space_style */
     widget = create_combo (SCIM_ANTHY_CONFIG_SPACE_TYPE,
                            (gpointer) &space_types,
-                           table, 4);
+                           GTK_TABLE (table), 4);
 
     /* ten key_style */
     widget = create_combo (SCIM_ANTHY_CONFIG_TEN_KEY_TYPE,
                            (gpointer) &ten_key_types,
-                           table, 5);
+                           GTK_TABLE (table), 5);
 
     /* behavior on period */
     widget = create_combo (SCIM_ANTHY_CONFIG_BEHAVIOR_ON_PERIOD,
                            (gpointer) &behavior_on_period,
-                           table, 6);
+                           GTK_TABLE (table), 6);
 
     return vbox;
 }
@@ -786,8 +980,8 @@ create_learning_page ()
 
     for (unsigned int i = 0; config_keyboards_reverse_learning[i].key; i++) {
         StringConfigData *entry = &config_keyboards_reverse_learning[i];
-        create_entry (entry, GTK_TABLE (table), i);
-        gtk_entry_set_editable (GTK_ENTRY (entry->widget), FALSE);
+        widget = create_entry (entry, GTK_TABLE (table), i);
+        gtk_entry_set_editable (GTK_ENTRY (widget), FALSE);
         button = gtk_button_new_with_label ("...");
         gtk_widget_show (button);
         gtk_table_attach (GTK_TABLE (table), button, 2, 3, i, i + 1,
@@ -842,15 +1036,31 @@ create_dict_page (void)
 static GtkWidget *
 create_candidates_window_page (void)
 {
-    GtkWidget *vbox, *widget;
+    GtkWidget *vbox, *widget, *table;
 
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
 
+    /* show candidates label */
+    widget = create_check_button (SCIM_ANTHY_CONFIG_SHOW_CANDIDATES_LABEL);
+    gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 2);
+
     /* close candidate window on select */
     widget = create_check_button (SCIM_ANTHY_CONFIG_CLOSE_CAND_WIN_ON_SELECT);
     gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 2);
 
+    table = gtk_table_new (2, 2, FALSE);
+    gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+    gtk_widget_show (table);
+
+    /* number of triggers until show candidates window */
+    create_spin_button (SCIM_ANTHY_CONFIG_CAND_WIN_PAGE_SIZE,
+                        GTK_TABLE (table), 0);
+
+    /* number of triggers until show candidates window */
+    create_spin_button (SCIM_ANTHY_CONFIG_N_TRIGGERS_TO_SHOW_CAND_WIN,
+                        GTK_TABLE (table), 1);
+
     return vbox;
 }
 
@@ -866,10 +1076,10 @@ create_toolbar_page (void)
     widget = create_check_button (SCIM_ANTHY_CONFIG_SHOW_INPUT_MODE_LABEL);
     gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 2);
 
-    widget = create_check_button (SCIM_ANTHY_CONFIG_SHOW_CONVERSION_MODE_LABEL);
+    widget = create_check_button (SCIM_ANTHY_CONFIG_SHOW_TYPING_METHOD_LABEL);
     gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 2);
 
-    widget = create_check_button (SCIM_ANTHY_CONFIG_SHOW_TYPING_METHOD_LABEL);
+    widget = create_check_button (SCIM_ANTHY_CONFIG_SHOW_CONVERSION_MODE_LABEL);
     gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 2);
 
     widget = create_check_button (SCIM_ANTHY_CONFIG_SHOW_PERIOD_STYLE_LABEL);
@@ -906,6 +1116,72 @@ create_toolbar_page (void)
     return vbox;
 }
 
+static GtkWidget *
+create_appearance_page (void)
+{
+    GtkWidget *vbox, *table, *omenu, *widget, *hbox; 
+
+    vbox = gtk_vbox_new (FALSE, 0);
+    gtk_widget_show (vbox);
+
+    table = gtk_table_new (2, 3, FALSE);
+    gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+    gtk_widget_show (table);
+
+    /* preedit style */
+    omenu = create_option_menu (SCIM_ANTHY_CONFIG_PREEDIT_STYLE,
+                                &preedit_style, GTK_TABLE (table), 0);
+
+    /* preedit color */
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    widget = create_color_button (SCIM_ANTHY_CONFIG_PREEDIT_FG_COLOR);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
+    gtk_table_attach (GTK_TABLE (table), hbox, 2, 3, 0, 1,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL), 4, 4);
+    gtk_widget_set_sensitive (hbox, FALSE);
+
+    g_signal_connect ((gpointer) GTK_OPTION_MENU (omenu), "changed",
+                      G_CALLBACK (on_preedit_style_menu_changed), hbox);
+
+    /* conversion style */
+    omenu = create_option_menu (SCIM_ANTHY_CONFIG_CONVERSION_STYLE,
+                                &preedit_style, GTK_TABLE (table), 1);
+
+    /* conversion color */
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    widget = create_color_button (SCIM_ANTHY_CONFIG_CONVERSION_FG_COLOR);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
+    gtk_table_attach (GTK_TABLE (table), hbox, 2, 3, 1, 2,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL), 4, 4);
+    gtk_widget_set_sensitive (hbox, FALSE);
+
+    g_signal_connect ((gpointer) GTK_OPTION_MENU (omenu), "changed",
+                      G_CALLBACK (on_preedit_style_menu_changed), hbox);
+
+    /* selected segment style */
+    omenu = create_option_menu (SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_STYLE,
+                                &preedit_style, GTK_TABLE (table), 2);
+
+    /* selected segment color */
+    hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (hbox);
+    widget = create_color_button (SCIM_ANTHY_CONFIG_SELECTED_SEGMENT_FG_COLOR);
+    gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 0);
+    gtk_table_attach (GTK_TABLE (table), hbox, 2, 3, 2, 3,
+                      (GtkAttachOptions) (GTK_FILL),
+                      (GtkAttachOptions) (GTK_FILL), 4, 4);
+    gtk_widget_set_sensitive (hbox, FALSE);
+
+    g_signal_connect ((gpointer) GTK_OPTION_MENU (omenu), "changed",
+                      G_CALLBACK (on_preedit_style_menu_changed), hbox);
+
+    return vbox;
+}
+
 static GtkWidget *
 create_setup_window (void)
 {
@@ -924,7 +1200,7 @@ create_setup_window (void)
         gtk_widget_show (label);
         gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
-        // Create the key bind pages.
+        // Create the key bind page.
         page = create_keyboard_page ();
         label = gtk_label_new (_("Key bindings"));
         gtk_widget_show (label);
@@ -932,7 +1208,13 @@ create_setup_window (void)
 
         // Create the romaji page.
         page = romaji_page_create_ui ();
-        label = gtk_label_new (_("Romaji Typing"));
+        label = gtk_label_new (_("Romaji typing"));
+        gtk_widget_show (label);
+        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+
+        // Create the kana page.
+        page = kana_page_create_ui ();
+        label = gtk_label_new (_("Kana typing"));
         gtk_widget_show (label);
         gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
@@ -960,6 +1242,12 @@ create_setup_window (void)
         gtk_widget_show (label);
         gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
+        // Create the appearance  page.
+        page = create_appearance_page ();
+        label = gtk_label_new (_("Appearance"));
+        gtk_widget_show (label);
+        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+
         // for preventing enabling left arrow.
         gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 1);
         gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
@@ -994,6 +1282,21 @@ setup_combo_value (GtkCombo *combo, const String & str)
         gtk_entry_set_text (GTK_ENTRY (combo->entry), defval);
 }
 
+static void
+setup_option_menu_value (GtkOptionMenu *omenu, const String & str)
+{
+    ComboConfigCandidate *data
+        = static_cast<ComboConfigCandidate*>
+        (g_object_get_data (G_OBJECT (omenu), DATA_POINTER_KEY));
+
+    for (unsigned int i = 0; data[i].label; i++) {
+        if (!strcmp (data[i].data, str.c_str ())) {
+            gtk_option_menu_set_history (omenu, i);
+            return;
+        }
+    }
+}
+
 static void
 setup_key_theme_menu (GtkOptionMenu *omenu)
 {
@@ -1035,7 +1338,9 @@ setup_key_theme_menu (GtkOptionMenu *omenu)
 
     gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), 1);
 
-    if (__config_key_theme == "User defined") {
+    if (__config_key_theme_file == __user_style_file.get_file_name () ||
+        __config_key_theme      == __user_style_file.get_title ())
+    {
         gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), 0);
 
     } else {
@@ -1046,7 +1351,7 @@ setup_key_theme_menu (GtkOptionMenu *omenu)
         {
             gint idx = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (node->data),
                                                            INDEX_KEY));
-            if (__style_list[idx].get_title () == __config_key_theme) {
+            if (__style_list[idx].get_file_name () == __config_key_theme_file) {
                 gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), i);
                 break;
             }
@@ -1068,13 +1373,24 @@ setup_widget_value (void)
                                           entry.value);
     }
 
+    for (unsigned int i = 0; config_int_common[i].key; i++) {
+        IntConfigData &entry = config_int_common[i];
+        if (entry.widget)
+            gtk_spin_button_set_value (GTK_SPIN_BUTTON (entry.widget),
+                                       entry.value);
+    }
+
     for (unsigned int i = 0; config_string_common[i].key; i++) {
         StringConfigData &entry = config_string_common[i];
-        if (entry.widget && GTK_IS_COMBO (entry.widget))
+        if (entry.widget && GTK_IS_OPTION_MENU (entry.widget)) {
+            setup_option_menu_value (GTK_OPTION_MENU (entry.widget),
+                                     entry.value);
+        } else if (entry.widget && GTK_IS_COMBO (entry.widget)) {
             setup_combo_value (GTK_COMBO (entry.widget), entry.value);
-        else if (entry.widget && GTK_IS_ENTRY (entry.widget))
+        } else if (entry.widget && GTK_IS_ENTRY (entry.widget)) {
             gtk_entry_set_text (GTK_ENTRY (entry.widget),
                                 entry.value.c_str ());
+        }
     }
 
     for (unsigned int j = 0; j < __key_conf_pages_num; j++) {
@@ -1094,6 +1410,14 @@ setup_widget_value (void)
                 config_keyboards_reverse_learning[i].value.c_str ());
         }
     }
+    
+    for (unsigned int i = 0; config_color_common[i].fg_key; i++) {
+        ColorConfigData &entry = config_color_common[i];
+        if (entry.widget) {
+            scim_color_button_set_colors (SCIM_COLOR_BUTTON (entry.widget),
+                                          entry.fg_value, entry.bg_value);
+        }
+    }
 
     gtk_option_menu_set_history
         (GTK_OPTION_MENU (__widget_key_categories_menu), 0);
@@ -1157,12 +1481,20 @@ load_config (const ConfigPointer &config)
     __config_key_theme
         = config->read (String (SCIM_ANTHY_CONFIG_KEY_THEME),
                         String (SCIM_ANTHY_CONFIG_KEY_THEME_DEFAULT));
+    __config_key_theme_file
+        = config->read (String (SCIM_ANTHY_CONFIG_KEY_THEME_FILE),
+                        String (SCIM_ANTHY_CONFIG_KEY_THEME_FILE_DEFAULT));
 
     for (unsigned int i = 0; config_bool_common[i].key; i++) {
         BoolConfigData &entry = config_bool_common[i];
         entry.value = config->read (String (entry.key), entry.value);
     }
 
+    for (unsigned int i = 0; config_int_common[i].key; i++) {
+        IntConfigData &entry = config_int_common[i];
+        entry.value = config->read (String (entry.key), entry.value);
+    }
+
     for (unsigned int i = 0; config_string_common[i].key; i++) {
         StringConfigData &entry = config_string_common[i];
         entry.value = config->read (String (entry.key), entry.value);
@@ -1181,13 +1513,23 @@ load_config (const ConfigPointer &config)
         entry.value = config->read (String (entry.key), entry.value);
     }
 
+    for (unsigned int i = 0; config_color_common[i].fg_key; i++) {
+        ColorConfigData &entry = config_color_common[i];
+        entry.fg_value = config->read (String (entry.fg_key), entry.fg_value);
+        entry.bg_value = config->read (String (entry.bg_key), entry.bg_value);
+    }
+
     romaji_page_load_config (config);
+    kana_page_load_config (config);
 
     setup_widget_value ();
 
     for (unsigned int i = 0; config_bool_common[i].key; i++)
         config_bool_common[i].changed = false;
 
+    for (unsigned int i = 0; config_int_common[i].key; i++)
+        config_int_common[i].changed = false;
+
     for (unsigned int i = 0; config_string_common[i].key; i++)
         config_string_common[i].changed = false;
 
@@ -1199,6 +1541,9 @@ load_config (const ConfigPointer &config)
     for (unsigned int i = 0; config_keyboards_reverse_learning[i].key; i++)
         config_keyboards_reverse_learning[i].changed = false;
 
+    for (unsigned int i = 0; config_color_common[i].fg_key; i++)
+        config_color_common[i].changed = false;
+
     __config_changed = false;
 }
 
@@ -1211,6 +1556,9 @@ save_config (const ConfigPointer &config)
     __config_key_theme
         = config->write (String (SCIM_ANTHY_CONFIG_KEY_THEME),
                          String (__config_key_theme));
+    __config_key_theme_file
+        = config->write (String (SCIM_ANTHY_CONFIG_KEY_THEME_FILE),
+                         String (__config_key_theme_file));
 
     for (unsigned int i = 0; config_bool_common[i].key; i++) {
         BoolConfigData &entry = config_bool_common[i];
@@ -1219,6 +1567,13 @@ save_config (const ConfigPointer &config)
         entry.changed = false;
     }
 
+    for (unsigned int i = 0; config_int_common[i].key; i++) {
+        IntConfigData &entry = config_int_common[i];
+        if (entry.changed)
+            entry.value = config->write (String (entry.key), entry.value);
+        entry.changed = false;
+    }
+
     for (unsigned int i = 0; config_string_common[i].key; i++) {
         StringConfigData &entry = config_string_common[i];
         if (entry.changed)
@@ -1242,6 +1597,15 @@ save_config (const ConfigPointer &config)
         entry.changed = false;
     }
 
+    for (unsigned int i = 0; config_color_common[i].fg_key; i++) {
+        ColorConfigData &entry = config_color_common[i];
+        if (entry.changed) {
+            entry.fg_value = config->write (String (entry.fg_key), entry.fg_value);
+            entry.bg_value = config->write (String (entry.bg_key), entry.bg_value);
+        }
+        entry.changed = false;
+    }
+
     __config_changed = false;
 
     if (__style_changed) {
@@ -1251,6 +1615,7 @@ save_config (const ConfigPointer &config)
     }
 
     romaji_page_save_config (config);
+    kana_page_save_config (config);
 }
 
 static bool
@@ -1259,7 +1624,8 @@ query_changed (void)
     return
         __config_changed ||
         __style_changed ||
-        romaji_page_query_changed ();
+        romaji_page_query_changed () ||
+        kana_page_query_changed ();
 }
 
 
@@ -1276,6 +1642,18 @@ on_default_toggle_button_toggled (GtkToggleButton *togglebutton,
     }
 }
 
+static void
+on_default_spin_button_changed (GtkSpinButton *spinbutton, gpointer user_data)
+{
+    IntConfigData *entry = static_cast<IntConfigData*> (user_data);
+
+    if (entry) {
+        entry->value = static_cast<int> (gtk_spin_button_get_value (spinbutton));
+        entry->changed = true;
+        __config_changed = true;
+    }
+}
+
 static void
 on_default_editable_changed (GtkEditable *editable,
                              gpointer     user_data)
@@ -1289,7 +1667,7 @@ on_default_editable_changed (GtkEditable *editable,
     }
 }
 
-static void
+void
 on_default_key_selection_clicked (GtkButton *button,
                                   gpointer   user_data)
 {
@@ -1335,14 +1713,56 @@ on_default_combo_changed (GtkEditable *editable,
 
     for (unsigned int i = 0; data[i].label; i++) {
         if (label && !strcmp (_(data[i].label), label)) {
-            entry->value = data[i].data;
-            entry->changed = true;
+            entry->value     = data[i].data;
+            entry->changed   = true;
             __config_changed = true;
             break;
         }
     }
 }
 
+static void
+on_default_option_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
+{
+    StringConfigData *entry = static_cast<StringConfigData*> (user_data);
+    ComboConfigCandidate *data = static_cast<ComboConfigCandidate*>
+        (g_object_get_data (G_OBJECT (omenu),
+                            DATA_POINTER_KEY));
+
+    if (!entry) return;
+    if (!data) return;
+
+    for (int i = 0; data[i].label; i++) {
+        if (i == gtk_option_menu_get_history (omenu)) {
+            entry->value     = data[i].data;
+            entry->changed   = true;
+            __config_changed = true;
+            break;
+        }
+    }
+}
+
+static void
+on_preedit_style_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
+{
+    GtkWidget *widget = GTK_WIDGET (user_data);
+
+    gint idx = gtk_option_menu_get_history (omenu);
+
+    for (int i = 0; preedit_style[i].data && i <= idx; i++) {
+        if (i == idx &&
+            (!strcmp (preedit_style[i].data, "Color") ||
+             !strcmp (preedit_style[i].data, "FGColor") ||
+             !strcmp (preedit_style[i].data, "BGColor")))
+        {
+            gtk_widget_set_sensitive (widget, TRUE);
+            return;
+        }
+    }
+
+    gtk_widget_set_sensitive (widget, FALSE);
+}
+
 static void
 on_dict_menu_label_toggled (GtkToggleButton *togglebutton,
                             gpointer         user_data)
@@ -1416,7 +1836,8 @@ on_key_theme_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
 
     // set new key bindings
     if (idx == 0) {
-        __config_key_theme = "User defined";
+        __config_key_theme      = String ("User defined");
+        __config_key_theme_file = String (""); //__user_style_file.get_file_name ();
 
     } else if (idx == 1) {
         for (unsigned int j = 0; j < __key_conf_pages_num; j++) {
@@ -1425,7 +1846,8 @@ on_key_theme_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
                     = __key_conf_pages[j].data[i].default_value;
             }
         }
-        __config_key_theme = "Default";
+        __config_key_theme      = String ("Default");
+        __config_key_theme_file = String ("");
 
     } else if (theme_idx >= 0) {
         // reset key bindings
@@ -1446,7 +1868,8 @@ on_key_theme_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
                 std::cerr << "No entry for : " << key << std::endl;
             }
         }
-        __config_key_theme = __style_list[theme_idx].get_title ();
+        __config_key_theme      = __style_list[theme_idx].get_title ();
+        __config_key_theme_file = __style_list[theme_idx].get_file_name ();
     }
 
     // sync widgets
@@ -1567,7 +1990,20 @@ on_dict_launch_button_clicked (GtkButton *button, gpointer user_data)
     if (entry->widget) {
         const char *command = gtk_entry_get_text (GTK_ENTRY (entry->widget));
         if (command && *command)
-            launch_program (command);
+            util_launch_program (command);
+    }
+}
+
+static void
+on_color_button_changed (ScimColorButton *button,
+                         gpointer         user_data)
+{
+    ColorConfigData *entry = static_cast <ColorConfigData*> (user_data);
+
+    if (entry->widget) {
+        scim_color_button_get_colors (button, &entry->fg_value, &entry->bg_value);
+        entry->changed = true;
+        __config_changed = true;
     }
 }
 
